<?php

namespace Drupal\vipps_mobilepay_commerce\Resolver;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_price\MinorUnitsConverterInterface;
use Drupal\commerce_shipping\ShipmentManagerInterface;
use Drupal\commerce_shipping\ShippingOrderManagerInterface;
use Drupal\profile\Entity\ProfileInterface;
use zaporylie\Vipps\Model\Checkout\v3\Amount;
use zaporylie\Vipps\Model\Checkout\v3\OtherLogisticsOption;

/**
 * Returns the site's default remote order id.
 */
class CommerceShippingMethodsResolver {

  /**
   * The shipping order manager.
   *
   * @var \Drupal\commerce_shipping\ShippingOrderManagerInterface
   */
  protected $orderManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The shipment manager.
   *
   * @var \Drupal\commerce_shipping\ShipmentManagerInterface
   */
  protected $shipmentManager;

  /**
   * The minor units converter.
   *
   * @var \Drupal\commerce_price\MinorUnitsConverterInterface
   */
  protected $minorUnitsConverter;

  /**
   * CommerceShippingMethodsResolver constructor.
   *
   * @param \Drupal\commerce_shipping\ShippingOrderManagerInterface $orderManager
   *   The Shipping Order Manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\commerce_shipping\ShipmentManagerInterface $shipment_manager
   *   The shipment manager.
   * @param \Drupal\commerce_price\MinorUnitsConverterInterface $minor_units_converter
   *   The minor units converter.
   */
  public function __construct(ShippingOrderManagerInterface $orderManager, EntityTypeManagerInterface $entityTypeManager, ShipmentManagerInterface $shipment_manager, MinorUnitsConverterInterface $minor_units_converter) {
    $this->orderManager = $orderManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->shipmentManager = $shipment_manager;
    $this->minorUnitsConverter = $minor_units_converter;
  }

  /**
   * {@inheritdoc}
   */
  public function resolve(OrderInterface $order) {

    $profiles = $order->collectProfiles();
    if (isset($profiles['shipping']) && ($profiles['shipping'] instanceof ProfileInterface)) {
      $profile = $profiles['shipping'];
    }
    else {
      $profile = $this->orderManager->createProfile($order, [
        'address' => [
          'country_code' => '',
          'postal_code' => '',
          'locality' => '',
          'address_line1' => '',
          'address_line2' => '',
        ],
      ]);
    }

    $i = 0;
    /** @var \Drupal\commerce_shipping\Entity\Shipment[] $shipments */
    $shipments = $this->orderManager->pack($order, $profile);
    $available_shipping_methods = [];
    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
    foreach ($shipments as $shipment) {
      $shipping_rates = $this->shipmentManager->calculateRates($shipment);
      foreach ($shipping_rates as $shipping_rate) {
        // @see \Drupal\commerce_shipping\Plugin\Field\FieldWidget\ShippingRateWidget::formElement().
        $service = $shipping_rate->getService();
        $amount = $shipping_rate->getAmount();

        $option_id = $shipping_rate->getShippingMethodId() . '--' . $service->getId();
        $option_label = $service->getLabel();
        $shipping_details = new OtherLogisticsOption();
        $shipping_details->setAmount((new Amount())
          ->setCurrency($amount->getCurrencyCode())
          ->setValue($this->minorUnitsConverter->toMinorUnits($amount)));
        $shipping_details->setId($option_id);
        $shipping_details->setPriority($i);
        $shipping_details->setTitle($option_label);
        $shipping_details->setBrand('OTHER');
        $i++;

        $available_shipping_methods[] = $shipping_details;
      }
    }

    return $available_shipping_methods;
  }

}
